PROGRAM MAIN

  USE GLOBAL, ONLY : WP, W0

  IMPLICIT NONE

!----------------------------------------------------------------------

  CHARACTER (20) :: S1_NAME, S2_NAME
  
  INTEGER        :: I, L, MT, NF_TF
  REAL(WP)       :: DT, FMIN, FMAX, FF
  
  REAL(WP), DIMENSION (:)  , ALLOCATABLE :: S1, S2

  REAL(WP)                               ::   EM,   PM
  REAL(WP), DIMENSION (:  ), ALLOCATABLE ::  TEM,  TPM
  REAL(WP), DIMENSION (:  ), ALLOCATABLE ::  FEM,  FPM
  REAL(WP), DIMENSION (:,:), ALLOCATABLE :: TFEM, TFPM, CWT_1, CWT_2
  
!----------------------------------------------------------------------

  INTERFACE 

    SUBROUTINE TF_MISFITS (S1, S2,   DT, MT, FMIN, FMAX, NF_TF,        &
                                    TFEM,  TFPM,                       &
                                     TEM,   TPM,                       &
                                     FEM,   FPM,                       &
                                      EM,    PM,  CWT_1, CWT_2 )

      USE GLOBAL, ONLY: WP
      IMPLICIT NONE
      INTEGER , INTENT(IN) :: MT, NF_TF
      REAL(WP), INTENT(IN) :: DT, FMIN, FMAX
      REAL(WP), DIMENSION (1:MT)   , INTENT(IN)  :: S1, S2
      REAL(WP),                      INTENT(OUT) ::  EM,  PM
      REAL(WP), DIMENSION (1:MT   ), INTENT(OUT) :: TEM, TPM
      REAL(WP), DIMENSION (1:NF_TF), INTENT(OUT) :: FEM, FPM
      REAL(WP), DIMENSION (1:MT,1:NF_TF), INTENT(OUT) ::  TFEM, TFPM,  &
                                                          CWT_1, CWT_2
    END SUBROUTINE TF_MISFITS

  END INTERFACE

!----------------------------------------------------------------------

  NAMELIST /INPUT/   MT, DT, S1_NAME, S2_NAME, FMIN, FMAX

!----------------------------------------------------------------------

  FMIN = -999999.
  FMAX = -999999.
  
  OPEN ( 10, FILE='HF_TF-MISFITS_NR', STATUS = 'OLD' )
  READ ( 10, NML = INPUT )

!=== ASSIGNMENT OF DEFAULT VALUES FOR FMIN AND FMAX  ( IF NOT PROVIDED )

  IF ( FMIN == -999999. ) THEN
    FMIN = 1./REAL(MT)/DT
  END IF
  IF ( FMAX == -999999. ) THEN
    FMAX = 1./2./DT
  END IF

!===================== COMPUTATION OF THE NUMBER OF SAMPLES IN FREQUENCY

  FF    = 1.+1./SQRT(2.)/W0
  NF_TF = 1 + INT( LOG(FMAX/FMIN) / LOG(FF) )

!=============================================== ALLOCATION OF VARIABLES 
    
  ALLOCATE ( S1(MT), S2(MT) )
  
  ALLOCATE ( TEM (1:MT), TPM (1:MT), FEM (1:NF_TF), FPM (1:NF_TF),     &
             TFEM(1:MT,1:NF_TF)              , TFPM(1:MT,1:NF_TF),     &
                         CWT_1(1:MT,1:NF_TF), CWT_2(1:MT,1:NF_TF) )

!============================== READ INPUT FILES WITH SIGNALS S1 AND S2

  OPEN (21, FILE=S1_NAME, STATUS='OLD')
  OPEN (22, FILE=S2_NAME, STATUS='OLD')

  DO I = 1, MT
    READ (21, *) S1(I)
    READ (22, *) S2(I)
  END DO

  CLOSE (21)
  CLOSE (22)

!======================= CALL SUBROUTINE FOR THE COMPUTATIONS OF MISFITS

  CALL TF_MISFITS      (S1, S2, DT, MT, FMIN, FMAX, NF_TF,             &
                                TFEM,  TFPM,                           &
                                 TEM,   TPM,                           &
                                 FEM,   FPM,                           &
                                  EM,    PM,   CWT_1, CWT_2  )

!========================================================= WRITE RESULTS

  OPEN  ( 21, FILE='MISFITS.DAT', STATUS='NEW' )
  WRITE ( 21, *) FMIN, FMAX, NF_TF, MT, DT
  WRITE ( 21, *) EM, PM
  CLOSE ( 21 )

  OPEN  ( 21, FILE='TFEM.DAT', STATUS='NEW' )
  DO L = 1, NF_TF
    WRITE ( 21, * ) ( TFEM(I,L), I = 1, MT )
  END DO
  CLOSE ( 21 )

  OPEN ( 21, FILE='TFPM.DAT', STATUS='NEW' )
  DO L = 1, NF_TF
    WRITE ( 21, * ) ( TFPM(I,L), I = 1, MT )
  END DO
  CLOSE ( 21 )

  OPEN ( 21, FILE='TEM.DAT', STATUS='NEW' )
  DO I = 1, MT
    WRITE ( 21, * ) TEM(I)
  END DO
  CLOSE ( 21 )

  OPEN ( 21, FILE='TPM.DAT', STATUS='NEW' )
  DO I = 1, MT
    WRITE ( 21, * ) TPM(I)
  END DO
  CLOSE ( 21 )

  OPEN ( 21, FILE='FEM.DAT', STATUS='NEW' )
  DO L = 1, NF_TF
    WRITE ( 21, * ) FEM(L)
  END DO
  CLOSE ( 21 )

  OPEN ( 21, FILE='FPM.DAT', STATUS='NEW' )
  DO L = 1, NF_TF
    WRITE ( 21, * ) FPM(L)
  END DO
  CLOSE ( 21 )

  OPEN ( 21, FILE='CWT_1.DAT', STATUS='NEW' )
  DO L = 1, NF_TF
    WRITE ( 21, * ) ( CWT_1(I,L), I = 1, MT )
  END DO
  CLOSE ( 21 )

  OPEN ( 21, FILE='CWT_2.DAT', STATUS='NEW' )
  DO L = 1, NF_TF
    WRITE ( 21, * ) ( CWT_2(I,L), I = 1, MT )
  END DO
  CLOSE ( 21 )


END PROGRAM MAIN