SUBROUTINE TF_MISFITS  (S1, S2, DT, MT, FMIN, FMAX, NF_TF,             &
                                TFEM,  TFPM,                           &
                                 TEM,   TPM,                           &
                                 FEM,   FPM,                           &
                                  EM,    PM,  CWT_1, CWT_2  )

  USE GLOBAL, ONLY: WP, PI, W0, KN, NF

  IMPLICIT NONE

!----------------------------------------------------------------------

  INTEGER , INTENT(IN) :: MT, NF_TF
  REAL(WP), INTENT(IN) :: DT, FMIN, FMAX
  
  REAL(WP), DIMENSION (1:MT)   , INTENT(IN)  :: S1, S2
  
  REAL(WP),                      INTENT(OUT) ::  EM, PM
  REAL(WP), DIMENSION (1:MT   ), INTENT(OUT) :: TEM, TPM
  REAL(WP), DIMENSION (1:NF_TF), INTENT(OUT) :: FEM, FPM

  REAL(WP), DIMENSION (1:MT,1:NF_TF), INTENT(OUT) ::  TFEM, TFPM,      &
                                                      CWT_1, CWT_2

  INTEGER            :: I, L

  REAL(WP)           :: NOMSE, NOMSP, DENOMS1, DENOMS2, DF, FF,        &
                        MAXDENOM1, MAXDENOM2, MAXTF
  
  REAL(WP), DIMENSION(1:MT,1:NF_TF)    :: DE , DP

  REAL(WP), DIMENSION(1:MAX(MT,NF_TF)) :: DET, DPT, DEF, DPF
                                          
  COMPLEX , DIMENSION(NF)           :: SS_1, SS_2
  COMPLEX , DIMENSION(1:MT,1:NF_TF) :: WV_1, WV_2

!----------------------------------------------------------------------

  INTERFACE 

    FUNCTION CWT(F_V, MT, NF_TF, DFA, FF, FMIN)
      USE GLOBAL, ONLY: WP
      IMPLICIT NONE
      INTEGER , INTENT(IN)    :: MT, NF_TF
      REAL(WP), INTENT(IN)    :: DFA, FF, FMIN
      COMPLEX , INTENT(INOUT), DIMENSION(:) :: F_V
      COMPLEX, DIMENSION (1:MT, 1:NF_TF) :: CWT
    END FUNCTION CWT
    
  END INTERFACE

!----------------------------------------------------------------------

! COMPUTATION OF CWT

  DF  = 1./(DT*REAL(NF))

  FF = EXP( LOG(FMAX/FMIN) / REAL(NF_TF - 1) )
   
  SS_1 = CMPLX(0.,0.)
  SS_2 = CMPLX(0.,0.)
  DO I = 1, MT
    SS_1 (I) = CMPLX( S1 (I), 0. )
    SS_2 (I) = CMPLX( S2 (I), 0. )
  END DO
  CALL FCOOLR (KN, SS_1, -1.)
  CALL FCOOLR (KN, SS_2, -1.)

  WV_1 (1:MT,1:NF_TF) = CWT(SS_1, MT ,NF_TF, DF, FF, FMIN)
  WV_2 (1:MT,1:NF_TF) = CWT(SS_2, MT, NF_TF, DF, FF, FMIN)

  CWT_1 = ABS(WV_1)
  CWT_2 = ABS(WV_2)
  
! COMPUTATION OF TFEM AND TFPM

  MAXTF = MIN ( MAXVAL (ABS (WV_1 (1:MT,1:NF_TF) ) ),                  &
                MAXVAL (ABS (WV_2 (1:MT,1:NF_TF) ) )  )
        
  DE = ABS(WV_1) - ABS(WV_2)
  DP = MIN(ABS(WV_1),ABS(WV_2)) * (ATAN2( IMAG(WV_1/WV_2) , REAL(WV_1/WV_2) )) / PI
   
  TFEM = DE / MAXTF
  TFPM = DP / MAXTF
        
! COMPUTATION OF AUXILIARY VARIABLES

  DO I = 1, MT
    DET  (I) = 0.
    DPT  (I) = 0.
    DO L = 1, NF_TF
      DET (I) = DET (I) +     DE(I,L)
      DPT (I) = DPT (I) +     DP(I,L)
    END DO
    DET (I) = DET (I)/REAL(NF_TF)
    DPT (I) = DPT (I)/REAL(NF_TF)
  END DO

  DO L = 1, NF_TF
    DEF   (L) = 0.
    DPF   (L) = 0.
    DO I = 1, MT
      DEF (L) = DEF (L) +     DE(I,L)
      DPF (L) = DPF (L) +     DP(I,L)
    END DO
    DEF (L) = DEF (L)/REAL(MT)
    DPF (L) = DPF (L)/REAL(MT)
  END DO  
  
! COMPUTATION OF TEM

  MAXDENOM1 = 0.
  MAXDENOM2 = 0.
  DO I = 1, MT
    DENOMS1 = 0.
    DENOMS2 = 0.
    DO L = 1, NF_TF
      DENOMS1 = DENOMS1 + ABS(WV_1(I,L))
      DENOMS2 = DENOMS2 + ABS(WV_2(I,L))
    END DO
    DENOMS1 = DENOMS1 / REAL(NF_TF)
    DENOMS2 = DENOMS2 / REAL(NF_TF)
    IF ( DENOMS1 > MAXDENOM1 ) MAXDENOM1 = DENOMS1
    IF ( DENOMS2 > MAXDENOM2 ) MAXDENOM2 = DENOMS2
  END DO

  DO I = 1, MT
    TEM(I) = DET (I) / MIN ( MAXDENOM1, MAXDENOM2 )
  END DO

! COMPUTATION OF TPM

  DO I = 1, MT
    TPM(I) = DPT (I) / MIN ( MAXDENOM1, MAXDENOM2 )
  END DO

! COMPUTATION OF FEM

  MAXDENOM1 = 0.
  MAXDENOM2 = 0.
  DO L = 1, NF_TF
    DENOMS1 = 0.
    DENOMS2 = 0.
    DO I = 1, MT
      DENOMS1 = DENOMS1 + ABS(WV_1(I,L))
      DENOMS2 = DENOMS2 + ABS(WV_2(I,L))
    END DO
    DENOMS1 = DENOMS1 / REAL(MT)
    DENOMS2 = DENOMS2 / REAL(MT)
    IF ( DENOMS1 > MAXDENOM1  ) MAXDENOM1  = DENOMS1
    IF ( DENOMS2 > MAXDENOM2  ) MAXDENOM2  = DENOMS2
  END DO

  DO L = 1, NF_TF
    FEM(L) = DEF (L) / MIN ( MAXDENOM1, MAXDENOM2 )
  END DO

! COMPUTATION OF FPM

  DO L = 1, NF_TF
    FPM(L) = DPF (L) / MIN ( MAXDENOM1, MAXDENOM2 )
  END DO

! COMPUTATION OF EM AND PM
 
  NOMSE  = 0.
  NOMSP  = 0.
  DENOMS1= 0.
  DENOMS2= 0.

  DO I = 1, MT
    DO L = 1, NF_TF

      NOMSE   = NOMSE   + ABS(DE   (I,L))*ABS(DE   (I,L))
      NOMSP   = NOMSP   + ABS(DP   (I,L))*ABS(DP   (I,L))
      DENOMS1 = DENOMS1 + ABS(WV_1 (I,L))*ABS(WV_1 (I,L))
      DENOMS2 = DENOMS2 + ABS(WV_2 (I,L))*ABS(WV_2 (I,L))

    END DO
  END DO
  
  EM = SQRT ( NOMSE / MIN (DENOMS1, DENOMS2) )
  PM = SQRT ( NOMSP / MIN (DENOMS1, DENOMS2) )


END SUBROUTINE  TF_MISFITS